<?php
/***********************************************
* File      :   badrequestexception.php
* Project   :   Z-Push
* Descr     :   Exception indicating that that some code is not
*               available which is non-fatal
*
* Created   :   16.1.2020
*
* Copyright 2007 - 2020 Zarafa Deutschland GmbH
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU Affero General Public License, version 3,
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU Affero General Public License for more details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
* Consult LICENSE file for details
************************************************/

class BadRequestException extends ZPushException {
    protected $defaultLogLevel = LOGLEVEL_WARN;
    protected $httpReturnCode = HTTP_CODE_400;
    protected $httpReturnMessage = "Bad Request";
}
